import datetime


class NewlyAddedVendorReport:
    def __init__(self, db, company_id):
        self.db = db
        self.company_id = company_id

    def build_saturn_report(self):
        results = self.db.get_results(
            query=f"SELECT * FROM c where c.type = 'vendorList' and c.employerId = '{self.company_id}' and c.vendorDates.added >= 1727762400 order by c.vendorDates.added desc")

        rows = []
        for item in results:
            if item['vendorDates']['added'] != '':

                item['vendorDates']['added'] = datetime.datetime.fromtimestamp(
                    item['vendorDates']['added']).strftime('%Y-%m-%d')
                rows.append({
                    'Vendor Name': item['vendorName'],
                    'Date Vendor Added': item['vendorDates']['added'],
                    'Vendor City': item['vendorAddress']['city'],
                    'Vendor Admin Name': item['vendorAccountAdmin']['name'],
                    'Vendor Admin Email': item['vendorAccountAdmin']['email'],
                    'Vendor Admin Phone': item['vendorAccountAdmin']['phone'],
                    'Rating': item['vendorOverallRating'],
                    'Current Deficiencies': '\n'.join(item['vendorOverallItems'])
                })

        return {
            'file_name': 'Added_Vendors.xlsx',
            'rows': rows,
            'headers': [x for x in rows[0].keys()] if len(rows) > 0 else []
        }



import csv


def generate_csv(column_list: list, file_name: str, data: list):
    """create csv file and write data to it"""
    csv_columns = column_list
    csv_file = file_name

    try:
        with open(csv_file, 'w', encoding='utf8') as csvfile:
            writer = csv.DictWriter(csvfile, fieldnames=csv_columns)
            writer.writeheader()
            for item in data:
                writer.writerow(item)
    except IOError:
        print('error')





class EmpQuestionsReport:
    def __init__(self, db, company_id: str) -> None:
        self.db = db
        self.companyId = company_id

    def format_questions(self, emp_questions: list) -> list:
        """Format questions"""
        results = []
        for item in emp_questions:
            if self.companyId in [x['id'] for x in item['employerList']]:
                results.append(item)

        return results

    def format_answers(self, vendor_list_items: list, vend_answers: list, emp_questions_formatted: list, emp_questions: list) -> list:
        """Format answers"""
        vendor_ids = [x['vendorId'] for x in vendor_list_items]
        emp_question_ids = [x['id'] for x in emp_questions_formatted]

        results = []
        for item in vend_answers:
            if (item['vendorId'] in vendor_ids) and (item['empQuestionId'] in emp_question_ids):
                item['inputType'] = [x['inputType']
                                     for x in emp_questions if x['id'] == item['empQuestionId']][0]
                item['questionText'] = [x['text']
                                        for x in emp_questions if x['id'] == item['empQuestionId']][0]
                results.append(item)

        return results

    def format_answers_by_question_type(self, vend_answers_formatted: list) -> list:
        """Format answers by question type"""
        for item in vend_answers_formatted:
            if item['inputType'] == 'mult_select':
                item['answer'] = str([x['text'] for x in item['answer'] if x['isActive'] is True]).replace(
                    '[', '').replace(']', '').replace('\'', '')

        return vend_answers_formatted

    def build_emp_questions_report(self) -> dict:
        """Build Tundra employee questions report"""
        vendor_list_items = self.db.get_results(
            query=f"SELECT c.vendorId, c.vendorName, c.employerId, c.employerName FROM c WHERE c.type = 'vendorList' AND c.employerId = '{self.companyId}'")

        emp_questions = self.db.get_results(
            query=f"SELECT * FROM c WHERE c.type = 'empQuestion'")

        emp_questions_formatted = self.format_questions(emp_questions)

        vend_answers = self.db.get_results(
            query=f"SELECT * FROM c WHERE c.type = 'vendAnswer'")

        vend_answers_formatted = self.format_answers(
            vendor_list_items, vend_answers, emp_questions_formatted, emp_questions)

        vend_answers_by_type = self.format_answers_by_question_type(
            vend_answers_formatted)

        rows = []
        for vendor_id in list(set([x['vendorId'] for x in vendor_list_items])):
            row_object = {'Vendor Name': [
                x['vendorName'] for x in vendor_list_items if x['vendorId'] == vendor_id][0]}
            for inner_item in vend_answers_by_type:
                if vendor_id == inner_item['vendorId']:
                    row_object[inner_item['questionText']
                               ] = inner_item['answer']

            if len(row_object) > 1:
                rows.append(row_object)

        return {
            'file_name': 'Questions_Report.csv',
            'headers': ['Vendor Name'] + [x['text'] for x in emp_questions_formatted],
            'rows': rows
        }



from datetime import datetime
import os


class SafetyTicketsReport:
    """Safety Tickets Report"""

    def __init__(self, db, company_id: str, b2c_helper) -> None:
        self.db = db
        self.company_id = company_id
        self.b2c_helper = b2c_helper

    def get_report_headers(self) -> list:
        """Get report headers"""

        docs = self.db.get_results(
            query="SELECT * FROM c WHERE c.type = 'doc' AND c.docClass = 'additionalRequirement' AND c.isUserTicket = true")

        results = []
        for doc in docs:
            if 'employerId' in doc:
                if doc['employerId'] == self.company_id:
                    results.append(
                        {'docType': doc['docType'], 'docRef': doc['docRef']})
                else:
                    pass
            else:
                results.append(
                    {'docType': doc['docType'], 'docRef': doc['docRef']})

        return results

    def remove_duplicates(self, active_docs: list):
        """Remove duplicates"""
        test = []
        for item in active_docs:
            for key in item.keys():
                test.append(key)

        distinct_list = list(set(test))

        test = []
        for item in active_docs:
            for inner in distinct_list:
                for key, val in item.items():
                    if key == inner:

                        item = {
                            'key': key,
                            'val': val
                        }

                        try:
                            item['val'] = datetime.strptime(val, '%Y-%m-%d')
                        except ValueError:
                            pass

                        test.append(item)

        filtered_list = [x for x in test if x['val'] != '']

        final = []
        for item in distinct_list:
            try:
                tested = datetime.strftime(
                    max([x['val'] for x in filtered_list if x['key'] == item]), '%Y-%m-%d')
                final.append({item: tested})
            except ValueError:
                final.append({item: ''})

        return final

    def prime(self, base_dict: dict, user_doc_questions: list, user_doc_ids_by_user: list):
        """Prime"""
        user_doc_questions = [
            x for x in user_doc_questions if x['userDocId'] in user_doc_ids_by_user]

        test = []
        for k, v in enumerate(base_dict.items()):
            for item in user_doc_questions:
                if (k > 1) and (item['docRef'] == v[1]['docRef']):
                    test.append(
                        ({f"{v[1]['docType']} {item['text']}": item['answer']}))

        return test

    def build_final(self, user_docs: list, header_list: list):
        """Build final"""
        results = []
        for item in header_list:
            results.append({'key': item, 'val': ''})
            for inner in user_docs:

                for k, v in inner.items():
                    if item == k:
                        results.append({'key': k, 'val': v})

        final_results = []
        test = list(set(x['key'] for x in results))
        for item in test:
            tested = [x['val'] for x in results if x['key'] == item]

            if len(tested) > 1:
                final_results.append({
                    item: tested[1]
                })

            else:
                final_results.append({
                    item: ''
                })

        return final_results

    def build_safety_ticket_report(self):
        """Build safety ticket report"""
        headers = self.get_report_headers()

        b2c_users = self.b2c_helper.get_users(self.company_id)

        users = [
            x for x in b2c_users if x[f'extension_{os.environ["B2C_EXTENSION_APP_CLIENT_ID"]}_UserRoles'] != '8amAdmin']

        user_docs = self.db.get_results(
            query=f"SELECT * FROM c WHERE c.type = 'userDoc' and ARRAY_CONTAINS({[x['id'] for x in users]}, c.userId)")

        user_doc_questions = self.db.get_results(
            query=f"SELECT * FROM c WHERE c.type = 'userDocQuestion' and ARRAY_CONTAINS({[x['id'] for x in user_docs]}, c.userDocId)")

        doc_questions = self.db.get_results(
            query=f"SELECT * FROM c WHERE c.type = 'docQuestion'")

        new_headers = []
        for item in headers:
            for inner in doc_questions:
                if item['docRef'] == inner['docRef']:
                    new_headers.append(f"{item['docType']} {inner['text']}")

        user_rows = []
        for user in users:
            user_doc_ids_by_user = [x['id']
                                    for x in user_docs if user['id'] == x['userId']]

            test = {**{'User Name': user['displayName'],
                    'User Id': user['id']}, **dict(list(enumerate(headers)))}

            test = self.prime(test, user_doc_questions, user_doc_ids_by_user)

            if test:
                docs = self.remove_duplicates(test)
                row = self.build_final(docs, new_headers)
                row[0] = {
                    'User Name': user['displayName']
                }
                result = {}
                for obj in row:
                    result.update(obj)
                user_rows.append(result)

        return {
            'file_name': 'Safety_Tickets_Report.csv',
            'headers': ['User Name'] + new_headers,
            'rows': user_rows
        }
